@@ -127,7 +127,7 @@ anjuta_project_property_get_type (void)
  * @type: Property value type
  * @flags: Property flags
  * @description: (transfer none): Property description
- * @property: (transfer full): Default property value
+ * @default_value: (transfer full): Default property value
  * @user_data: (allow-none) (transfer full): Optional user data
  *
  * Returns: (transfer full):
@@ -138,7 +138,7 @@ anjuta_project_property_info_new (const gchar *id,
                                   AnjutaProjectValueType type,
                                   AnjutaProjectPropertyFlags flags,
                                   const gchar *description,
-                                  AnjutaProjectProperty *property,
+                                  AnjutaProjectProperty *default_value,
                                   gpointer user_data)
 {
 	AnjutaProjectPropertyInfo *info = g_slice_new0(AnjutaProjectPropertyInfo);
@@ -148,9 +148,11 @@ anjuta_project_property_info_new (const gchar *id,
 	info->type = type;
 	info->flags = flags;
 	info->description = g_strdup (description);
-	info->property = property;
+	info->default_value = default_value;
 	info->user_data = user_data;
 
+	info->default_value->info = info;
+
 	return info;
 }
 
@@ -159,7 +161,7 @@ anjuta_project_property_info_copy (AnjutaProjectPropertyInfo *info)
 {
 	return anjuta_project_property_info_new (info->id, info->name, info->type,
 	                                         info->flags, info->description,
-	                                         info->property, info->user_data);
+	                                         info->default_value, info->user_data);
 }
 
 void
@@ -168,7 +170,7 @@ anjuta_project_property_info_free (AnjutaProjectPropertyInfo *info)
 	g_free (info->id);
 	g_free (info->name);
 	g_free (info->description);
-	anjuta_project_property_free (info->property);
+	anjuta_project_property_free (info->default_value);
 	g_slice_free (AnjutaProjectPropertyInfo, info);
 }
 
@@ -871,7 +873,7 @@ anjuta_project_node_get_property (AnjutaProjectNode *node, const gchar *id)
 		GList *found;
 
 		/* Get default property */
-		prop = info->property;
+		prop = info->default_value;
 
 		/* Find custom property */
 		found = g_list_find_custom (node->properties, info, find_property);
@@ -899,7 +901,7 @@ anjuta_project_node_get_map_property (AnjutaProjectNode *node, const gchar *id,
 		GList *found;
 
 		/* Get default property */
-		prop = info->property;
+		prop = info->default_value;
 
 		/* Find property */
 		found = node->properties;
@@ -939,6 +941,14 @@ anjuta_project_node_clear_state (AnjutaProjectNode *node, AnjutaProjectNodeState
 	return TRUE;
 }
 
+/**
+ * anjuta_project_node_insert_property_info:
+ * @node: (transfer none):
+ * @info: (transfer none):
+ *
+ * Returns: (transfer none):
+ */
+
 AnjutaProjectPropertyInfo *
 anjuta_project_node_insert_property_info (AnjutaProjectNode *node,
                                           AnjutaProjectPropertyInfo *info)
@@ -973,10 +983,9 @@ AnjutaProjectProperty *
 anjuta_project_node_remove_property (AnjutaProjectNode *node, AnjutaProjectProperty *prop)
 {
 	/* Search the exact property, useful for list property */
-	if (prop != prop->info->property)
+	if (prop != prop->info->default_value)
 	{
 		node->properties = g_list_remove (node->properties, prop);
-		prop->info = NULL;
 	}
 
 	return prop;
@@ -1090,9 +1099,7 @@ enum {
 	PROP_NAME,
 	PROP_FILE,
 	PROP_STATE,
-	PROP_TYPE,
-	PROP_PROPERTIES,
-	PROP_PROPERTIES_INFO
+	PROP_TYPE
 };
 
 
@@ -1144,6 +1151,12 @@ anjuta_project_node_finalize (GObject *object)
 
 	if (node->name != NULL) g_free (node->name);
 
+	g_list_free_full (node->properties, (GDestroyNotify) anjuta_project_property_free);
+	node->properties = NULL;
+
+	g_list_free (node->properties_info);
+	node->properties_info = NULL;
+
 	G_OBJECT_CLASS (anjuta_project_node_parent_class)->finalize (object);
 }
 
@@ -1167,12 +1180,6 @@ anjuta_project_node_get_gobject_property (GObject    *object,
 		case PROP_TYPE:
 			g_value_set_flags (value, anjuta_project_node_get_node_type (node));
 			break;
-		case PROP_PROPERTIES:
-			g_value_set_pointer (value, node->properties);
-			break;
-		case PROP_PROPERTIES_INFO:
-			g_value_set_pointer (value, node->properties_info);
-			break;
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 	}
@@ -1202,13 +1209,6 @@ anjuta_project_node_set_gobject_property (GObject      *object,
 		case PROP_TYPE:
 			node->type = g_value_get_flags (value);
 			break;
-		case PROP_PROPERTIES:
-			node->properties = g_value_get_pointer (value);
-			break;
-		/* XXX: We may need to copy this instead */
-		case PROP_PROPERTIES_INFO:
-			node->properties_info = g_value_get_pointer (value);
-			break;
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 	}
@@ -1294,31 +1294,6 @@ anjuta_project_node_class_init (AnjutaProjectNodeClass *klass)
 	g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_FILE,
 	                                 pspec);
 
-/**
- * AnjutaProjectNode:properties:
- *
- * type: GLib.List<Anjuta.ProjectProperty>
- * Transfer: none
- */
-	pspec = g_param_spec_pointer ("properties",
-	                              "Properties",
-	                              "The list of properties",
-	                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
-	g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_PROPERTIES,
-	                                 pspec);
-/**
- * AnjutaProjectNode:properties-info:
- *
- * Type: GLib.List<Anjuta.ProjectPropertyInfo>
- * Transfer: none
- */
-	pspec = g_param_spec_pointer ("properties-info",
-	                              "Properties info",
-	                              "The list of all possible properties informations",
-	                              G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
-	g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_PROPERTIES_INFO,
-	                                 pspec);
-
 }
 
 
