@@ -180,6 +180,7 @@ on_goto_key_pressed (GtkWidget* entry, GdkEventKey* event, SearchBox* search_box
 		case GDK_KEY_KP_Enter:
 		case GDK_KEY_BackSpace:
 		case GDK_KEY_Delete:
+		case GDK_KEY_Tab:
 		{
 			/* This is a number or enter which is ok */
 			break;
@@ -851,7 +852,7 @@ on_replace_activated (GtkWidget* widget, SearchBox* search_box)
 
 	successful_replace = search_box_replace (search_box, widget, TRUE);
 
-	if (!successful_replace)
+	if (successful_replace)
 	{
 		search_box_forward_search (search_box, widget);
 	}	
@@ -929,6 +930,7 @@ static void
 search_box_init (SearchBox *object)
 {
 	SearchBoxPrivate* private = GET_PRIVATE(object);
+	GList* focus_chain = NULL;
 	
 	/* Button images */
 	GtkWidget* close = 
@@ -936,6 +938,8 @@ search_box_init (SearchBox *object)
 	
 	/* Searching */
 	private->search_entry = gtk_entry_new();
+	gtk_widget_set_tooltip_text (private->search_entry,
+	                             _("Use the context menu of the \"Find\" icon for more search options"));
 	g_signal_connect_swapped (G_OBJECT (private->search_entry), "activate", 
 	                          G_CALLBACK (search_box_forward_search),
 	                          object);
@@ -1029,7 +1033,13 @@ search_box_init (SearchBox *object)
 	gtk_grid_attach (GTK_GRID (private->grid), private->previous_button, 2, 0, 1, 1);
 	gtk_grid_attach (GTK_GRID (private->grid), private->next_button, 3, 0, 1, 1);
 
-	gtk_grid_attach (GTK_GRID (private->grid), private->close_button, 4, 0, 1, 1);
+	gtk_grid_attach_next_to (GTK_GRID (private->grid),
+	                         private->close_button,
+	                         private->next_button,
+	                         GTK_POS_RIGHT, 1, 1);
+	gtk_widget_set_hexpand(private->close_button, TRUE);
+	gtk_widget_set_halign(private->close_button,
+	                      GTK_ALIGN_END);
 
 	/* Add Replace elements to search box on 2nd level */
 	gtk_grid_attach (GTK_GRID (private->grid), private->replace_entry, 1, 1, 1, 1);
@@ -1050,6 +1060,22 @@ search_box_init (SearchBox *object)
 	
 	/* Pack grid into search box */
 	gtk_box_pack_start (GTK_BOX(object), private->grid, TRUE, TRUE, 0);
+
+	/* Set focus chain */
+	focus_chain = g_list_prepend (focus_chain, private->search_entry);
+	focus_chain = g_list_prepend (focus_chain, private->replace_entry);
+	focus_chain = g_list_prepend (focus_chain, private->next_button);
+	focus_chain = g_list_prepend (focus_chain, private->previous_button);
+	focus_chain = g_list_prepend (focus_chain, private->replace_button);
+	focus_chain = g_list_prepend (focus_chain, private->replace_all_button);
+	focus_chain = g_list_prepend (focus_chain, private->goto_entry);
+	focus_chain = g_list_prepend (focus_chain, private->close_button);
+	focus_chain = g_list_prepend (focus_chain, private->search_entry);
+	focus_chain = g_list_reverse (focus_chain);
+	gtk_container_set_focus_chain (GTK_CONTAINER (private->grid),
+	                               focus_chain);
+	g_list_free (focus_chain);
+	
 	gtk_widget_show_all (GTK_WIDGET (object));
 	
 }
