@@ -35,11 +35,18 @@
 #include <libanjuta/anjuta-utils.h>
 #include <libanjuta/anjuta-pkg-config-chooser.h>
 #include <libanjuta/anjuta-tree-combo.h>
+#include <libanjuta/interfaces/ianjuta-project-chooser.h>
 
 #define ICON_SIZE 16
 
 #define GLADE_FILE  PACKAGE_DATA_DIR "/glade/pm_dialogs.ui"
 
+#define ANJUTA_MANUAL	"anjuta-manual"
+#define ADD_SOURCE_HELP	"anjuta-project-files"
+#define ADD_TARGET_HELP	"project-manager-target"
+#define ADD_PACKAGE_HELP "project-manager-module"
+
+
 /* Types
   *---------------------------------------------------------------------------*/
 
@@ -243,47 +250,6 @@ setup_nodes_treeview (GbfProjectView           *view,
 	gbf_project_view_set_cursor_to_iter (view, selected);
 }
 
-static void
-setup_nodes_combo_box (AnjutaTreeComboBox      *view,
-						GbfProjectModel	   *model,
-                  	    GtkTreePath            *root,
-						GtkTreeModelFilterVisibleFunc func,
-						gpointer               data,
-						GtkTreeIter            *selected)
-{
-	GtkTreeIter iter;
-
-	g_return_if_fail (view != NULL);
-	g_return_if_fail (model != NULL);
-
-	pm_setup_project_renderer (GTK_CELL_LAYOUT (view));
-
-	if ((func != NULL) || (root != NULL))
-	{
-		GtkTreeModel *filter;
-
-		filter = gtk_tree_model_filter_new (GTK_TREE_MODEL (model), root);
-		if (func != NULL)
-		{
-			gtk_tree_model_filter_set_visible_func (GTK_TREE_MODEL_FILTER (filter), func, data, NULL);
-		}
-		anjuta_tree_combo_box_set_model (view, filter);
-		g_object_unref (filter);
-		if (pm_convert_project_iter_to_model_iter (filter, &iter, selected))
-		{
-			anjuta_tree_combo_box_set_active_iter (view, &iter);
-		}
-	}
-	else
-    {
-		anjuta_tree_combo_box_set_model (view, GTK_TREE_MODEL (model));
-		if (selected)
-		{
-			anjuta_tree_combo_box_set_active_iter (view, selected);
-		}
-	}
-}
-
 static void
 entry_changed_cb (GtkEditable *editable, gpointer user_data)
 {
@@ -560,18 +526,6 @@ add_label (const gchar *display_name, const gchar *value, GtkWidget *table, gint
 	*position = *position + 1;
 }
 
-static gboolean
-is_project_node_but_shortcut (GtkTreeModel *model, GtkTreeIter *iter, gpointer user_data)
-{
-	GbfTreeData *data;
-
-	gtk_tree_model_get (GTK_TREE_MODEL (model), iter,
-		    GBF_PROJECT_MODEL_COLUMN_DATA, &data,
-		    -1);
-
-	return (data != NULL) && (data->shortcut == NULL) && (gbf_tree_data_get_node (data) != NULL);
-}
-
 static void
 update_properties (PropertiesTable *table)
 {
@@ -597,7 +551,7 @@ update_properties (PropertiesTable *table)
 		title = _("Project properties");
 		break;
 	case ANJUTA_PROJECT_GROUP:
-		title = _("Directory properties");
+		title = _("Folder properties");
 		break;
 	case ANJUTA_PROJECT_TARGET:
 		title = _("Target properties");
@@ -681,7 +635,7 @@ update_properties (PropertiesTable *table)
 			GtkWidget *entry;
 
 			prop = anjuta_project_node_get_property (table->node, info->id);
-			if (prop != prop->info->property)
+			if (prop != prop->info->default_value)
 			{
 				/* This property has been set, display it in the main part */
 				entry = add_entry (table->project->project, table->node, prop, table->main, &main_pos);
@@ -689,7 +643,7 @@ update_properties (PropertiesTable *table)
 			else
 			{
 				/* This property has not been set, hide it by default */
-				entry = add_entry (table->project->project, table->node, info->property, table->extra, &extra_pos);
+				entry = add_entry (table->project->project, table->node, info->default_value, table->extra, &extra_pos);
 				single = TRUE;
 			}
 
@@ -836,6 +790,7 @@ pm_project_create_properties_dialog (AnjutaPmProject *project, GtkWindow *parent
 	GtkWidget *dialog = NULL;
 	GtkBuilder *bxml;
 	GtkWidget *node_combo;
+	GtkTreeIter iter;
 
 	g_return_val_if_fail (data != NULL, NULL);
 
@@ -858,13 +813,15 @@ pm_project_create_properties_dialog (AnjutaPmProject *project, GtkWindow *parent
 	g_object_unref (bxml);
 
 	/* Add combo node selection */
-	setup_nodes_combo_box (ANJUTA_TREE_COMBO_BOX (node_combo),
-	                       gbf_project_view_get_model(ANJUTA_PLUGIN_PROJECT_MANAGER (table->project->plugin)->view),
-	                       NULL,
-	                       is_project_node_but_shortcut,
-	                       NULL,
-	                       selected);
-
+	ianjuta_project_chooser_set_project_model (IANJUTA_PROJECT_CHOOSER (node_combo),
+	                                           IANJUTA_PROJECT_MANAGER (table->project->plugin),
+	                                           ANJUTA_PROJECT_ROOT,
+	                                           NULL);
+	if (pm_convert_project_iter_to_model_iter (GTK_TREE_MODEL (anjuta_tree_combo_box_get_model (ANJUTA_TREE_COMBO_BOX (node_combo))),
+	                                           &iter, selected))
+	{
+		anjuta_tree_combo_box_set_active_iter (ANJUTA_TREE_COMBO_BOX (node_combo), &iter);
+	}
 	g_signal_connect (node_combo, "changed",
 					G_CALLBACK (on_node_changed),
 					table);
@@ -916,14 +873,13 @@ anjuta_pm_project_show_properties_dialog (ProjectManagerPlugin *plugin, GtkTreeI
 		{
 			selected = &iter;
 		}
-	}
+ 	}
 
 	if (selected)
 	{
 		GbfTreeData *data;
 
 		gtk_tree_model_get (GTK_TREE_MODEL (gbf_project_view_get_model (plugin->view)), selected, GBF_PROJECT_MODEL_COLUMN_DATA, &data, -1);
-
 		dialog_ptr = &data->properties_dialog;
 
 		if (*dialog_ptr != NULL)
@@ -936,9 +892,9 @@ anjuta_pm_project_show_properties_dialog (ProjectManagerPlugin *plugin, GtkTreeI
 			*dialog_ptr = pm_project_create_properties_dialog (
 				plugin->project,
 				GTK_WINDOW (plugin->project->plugin->shell),
-			    plugin->view,
-			    data,
-				selected);
+		    	plugin->view,
+		    	data,
+			    selected);
 			if (*dialog_ptr != NULL)
 			{
 				g_object_add_weak_pointer (G_OBJECT (*dialog_ptr), (gpointer *)dialog_ptr);
@@ -960,7 +916,6 @@ anjuta_pm_project_new_group (ProjectManagerPlugin *plugin, GtkWindow *parent, Gt
 	GtkBuilder *gui;
 	GtkWidget *dialog, *group_name_entry, *ok_button;
 	GtkWidget *groups_view;
-	GtkTreePath *root;
 	gint response;
 	gboolean finished = FALSE;
 	AnjutaProjectNode *new_group = NULL;
@@ -987,14 +942,12 @@ anjuta_pm_project_new_group (ProjectManagerPlugin *plugin, GtkWindow *parent, Gt
 	else
 		gtk_widget_set_sensitive (ok_button, FALSE);
 
-	root = gbf_project_model_get_project_root (gbf_project_view_get_model (plugin->view));
 	setup_nodes_treeview (GBF_PROJECT_VIEW (groups_view),
 	                        plugin->view,
-	                      	root,
+	                      	NULL,
 							parent_filter_func,
 							GINT_TO_POINTER (ANJUTA_PROJECT_GROUP),
 							selected);
-	gtk_tree_path_free (root);
 	gtk_widget_show (groups_view);
 
 	if (parent)
@@ -1059,7 +1012,7 @@ anjuta_pm_project_new_group (ProjectManagerPlugin *plugin, GtkWindow *parent, Gt
 
 AnjutaProjectNode*
 anjuta_pm_project_new_source (ProjectManagerPlugin *plugin,
-							GtkWindow           *parent,
+                              GtkWindow           *parent,
 							GtkTreeIter         *default_parent,
 							const gchar         *default_uri)
 {
@@ -1088,6 +1041,146 @@ anjuta_pm_project_new_source (ProjectManagerPlugin *plugin,
 		return NULL;
 }
 
+static void
+on_target_changed (GtkWidget *chooser, GtkWidget *button)
+{
+	GFile *file;
+
+	file = ianjuta_project_chooser_get_selected (IANJUTA_PROJECT_CHOOSER (chooser), NULL);
+	gtk_widget_set_sensitive (button, file != NULL);
+}
+
+GList*
+anjuta_pm_add_source_dialog (ProjectManagerPlugin *plugin,
+                             GtkWindow *top_window,
+                             GtkTreeIter *default_target,
+                             GFile *default_source)
+{
+	GtkBuilder *gui;
+	GtkWidget *dialog;
+	GtkWidget *ok_button;
+	GtkWidget *target_chooser;
+	GtkWidget *source_chooser;
+	gint response;
+	gboolean finished = FALSE;
+	GList *sources = NULL;
+
+	g_return_val_if_fail (plugin->project != NULL, NULL);
+
+	gui = load_interface ("add_source_dialog");
+	g_return_val_if_fail (gui != NULL, NULL);
+
+	/* get all needed widgets */
+	dialog = GTK_WIDGET (gtk_builder_get_object (gui, "add_source_dialog"));
+	target_chooser = GTK_WIDGET (gtk_builder_get_object (gui, "target_chooser"));
+	source_chooser = GTK_WIDGET (gtk_builder_get_object (gui, "source_chooser"));
+	ok_button = GTK_WIDGET (gtk_builder_get_object (gui, "ok_add_source_button"));
+
+	/* Fill target selection */
+	ianjuta_project_chooser_set_project_model (IANJUTA_PROJECT_CHOOSER (target_chooser),
+	                                           IANJUTA_PROJECT_MANAGER (plugin),
+	                                           ANJUTA_PROJECT_SOURCE,
+	                                           NULL);
+	if (default_target != NULL)
+	{
+		GtkTreeIter iter;
+		if (pm_convert_project_iter_to_model_iter (GTK_TREE_MODEL (anjuta_tree_combo_box_get_model (ANJUTA_TREE_COMBO_BOX (target_chooser))),
+	    	                                       &iter, default_target))
+		{
+			anjuta_tree_combo_box_set_active_iter (ANJUTA_TREE_COMBO_BOX (target_chooser), &iter);
+		}
+	}
+	g_signal_connect (target_chooser, "changed",
+					G_CALLBACK (on_target_changed),
+					ok_button);
+	on_target_changed (target_chooser, ok_button);
+
+	/* Selected default file */
+	if (default_source != NULL) gtk_file_chooser_set_file (GTK_FILE_CHOOSER (source_chooser), default_source, NULL);
+
+	if (top_window) {
+		gtk_window_set_transient_for (GTK_WINDOW (dialog), top_window);
+	}
+
+	/* execute dialog */
+	while (!finished) {
+		response = gtk_dialog_run (GTK_DIALOG (dialog));
+
+		switch (response)
+		{
+		case GTK_RESPONSE_OK:
+		{
+			GFile *target_file;
+			AnjutaProjectNode *target;
+			GSList *files;
+
+			target_file = ianjuta_project_chooser_get_selected (IANJUTA_PROJECT_CHOOSER (target_chooser), NULL);
+			target = gbf_project_view_get_node_from_file (plugin->view, ANJUTA_PROJECT_UNKNOWN, target_file);
+			files = gtk_file_chooser_get_files (GTK_FILE_CHOOSER (source_chooser));
+			if ((target != NULL) && (files != NULL))
+			{
+				GString *err_mesg = g_string_new (NULL);
+				GSList *item;
+
+				for (item = files; item != NULL; item = g_slist_next (item))
+				{
+					gchar *path = g_file_get_path ((GFile *)item->data);
+					GError *err = NULL;
+					AnjutaProjectNode *node;
+
+					node = anjuta_pm_project_add_source (plugin->project,
+					                                     target,
+					                                     NULL,
+					                                     path,
+					                                     &err);
+					sources = g_list_prepend (sources, node);
+					if (err) {
+						gchar *str = g_strdup_printf ("%s: %s\n",
+						                              path,
+						                              err->message);
+						g_string_append (err_mesg, str);
+						g_error_free (err);
+						g_free (str);
+					}
+					g_free (path);
+				}
+				if (err_mesg->str && strlen (err_mesg->str) > 0)
+				{
+					error_dialog (top_window, _("Cannot add source files"),
+							"%s", err_mesg->str);
+				}
+				else {
+					finished = TRUE;
+				}
+				g_string_free (err_mesg, TRUE);
+				g_slist_foreach (files, (GFunc)g_object_unref, NULL);
+				g_slist_free (files);
+			}
+			else
+			{
+				error_dialog (top_window, _("Cannot add source files"),
+						"%s", _("The selected node cannot contain source files."));
+			}
+			break;
+		}
+		case GTK_RESPONSE_HELP:
+			anjuta_util_help_display (GTK_WIDGET (dialog), ANJUTA_MANUAL, ADD_SOURCE_HELP);
+			break;
+		default:
+			finished = TRUE;
+			break;
+		}
+	}
+
+	/* destroy stuff */
+	gtk_widget_destroy (dialog);
+	g_object_unref (gui);
+
+	sources = g_list_reverse (sources);
+	return sources;
+}
+
+
 GList*
 anjuta_pm_project_new_multiple_source (ProjectManagerPlugin *plugin,
 								GtkWindow           *top_window,
@@ -1109,11 +1202,11 @@ anjuta_pm_project_new_multiple_source (ProjectManagerPlugin *plugin,
 
 	g_return_val_if_fail (plugin->project != NULL, NULL);
 
-	gui = load_interface ("add_source_dialog");
+	gui = load_interface ("new_source_dialog");
 	g_return_val_if_fail (gui != NULL, NULL);
 
 	/* get all needed widgets */
-	dialog = GTK_WIDGET (gtk_builder_get_object (gui, "add_source_dialog"));
+	dialog = GTK_WIDGET (gtk_builder_get_object (gui, "new_source_dialog"));
 	targets_view = GTK_WIDGET (gtk_builder_get_object (gui, "targets_view"));
 	source_file_tree = GTK_WIDGET (gtk_builder_get_object (gui, "source_file_tree"));
 	browse_button = GTK_WIDGET (gtk_builder_get_object (gui, "browse_button"));
@@ -1265,6 +1358,9 @@ anjuta_pm_project_new_multiple_source (ProjectManagerPlugin *plugin,
 			}
 			break;
 		}
+		case GTK_RESPONSE_HELP:
+			anjuta_util_help_display (GTK_WIDGET (dialog), ANJUTA_MANUAL, ADD_SOURCE_HELP);
+			break;
 		default:
 			gtk_list_store_clear (GTK_LIST_STORE (list));
 			finished = TRUE;
@@ -1343,7 +1439,6 @@ anjuta_pm_project_new_target (ProjectManagerPlugin *plugin,
 	GtkBuilder *gui;
 	GtkWidget *dialog, *target_name_entry, *ok_button;
 	GtkWidget *target_type_combo, *groups_view;
-	GtkTreePath *root;
 	GtkListStore *types_store;
 	GtkCellRenderer *renderer;
 	gint response;
@@ -1373,14 +1468,12 @@ anjuta_pm_project_new_target (ProjectManagerPlugin *plugin,
 	else
 		gtk_widget_set_sensitive (ok_button, FALSE);
 
-	root = gbf_project_model_get_project_root (gbf_project_view_get_model (plugin->view));
 	setup_nodes_treeview (GBF_PROJECT_VIEW (groups_view),
 	                        plugin->view,
-	                       	root,
+	                       	NULL,
 							parent_filter_func,
 							GINT_TO_POINTER (ANJUTA_PROJECT_TARGET),
 							default_group);
-	gtk_tree_path_free (root);
 	gtk_widget_show (groups_view);
 
 	/* setup target types combo box */
@@ -1466,6 +1559,9 @@ anjuta_pm_project_new_target (ProjectManagerPlugin *plugin,
 
 				break;
 			}
+			case GTK_RESPONSE_HELP:
+				anjuta_util_help_display (GTK_WIDGET (dialog), ANJUTA_MANUAL, ADD_TARGET_HELP);
+				break;
 			default:
 				finished = TRUE;
 				break;
@@ -1505,7 +1601,7 @@ anjuta_pm_project_new_module (ProjectManagerPlugin *plugin,
 	GtkBuilder *gui;
 	GtkWidget *dialog;
 	GtkWidget *ok_button, *new_button;
-	GtkWidget *targets_view;
+	GtkWidget *target_chooser;
 	GtkWidget *modules_view;
 	GtkTreePath *root;
 	gint response;
@@ -1520,20 +1616,17 @@ anjuta_pm_project_new_module (ProjectManagerPlugin *plugin,
 
 	/* get all needed widgets */
 	dialog = GTK_WIDGET (gtk_builder_get_object (gui, "add_module_dialog"));
-	targets_view = GTK_WIDGET (gtk_builder_get_object (gui, "module_targets_view"));
+	target_chooser = GTK_WIDGET (gtk_builder_get_object (gui, "module_targets_chooser"));
 	modules_view = GTK_WIDGET (gtk_builder_get_object (gui, "modules_view"));
 	new_button = GTK_WIDGET (gtk_builder_get_object (gui, "new_package_button"));
 	ok_button = GTK_WIDGET (gtk_builder_get_object (gui, "ok_module_button"));
 
-	root = gbf_project_model_get_project_root (gbf_project_view_get_model (plugin->view));
-	setup_nodes_treeview (GBF_PROJECT_VIEW (targets_view),
-	                        plugin->view,
-	                       	root,
-							parent_filter_func,
-							GINT_TO_POINTER (ANJUTA_PROJECT_MODULE),
-							default_target);
-	gtk_tree_path_free (root);
-	gtk_widget_show (targets_view);
+	/* Fill target selection */
+	ianjuta_project_chooser_set_project_model (IANJUTA_PROJECT_CHOOSER (target_chooser),
+	                                           IANJUTA_PROJECT_MANAGER (plugin),
+	                                           ANJUTA_PROJECT_MODULE,
+	                                           NULL);
+	gtk_widget_show (target_chooser);
 	root = gbf_project_model_get_project_root (gbf_project_view_get_model (plugin->view));
 	setup_nodes_treeview (GBF_PROJECT_VIEW (modules_view),
 	                        plugin->view,
@@ -1563,10 +1656,7 @@ anjuta_pm_project_new_module (ProjectManagerPlugin *plugin,
 		gtk_window_set_transient_for (GTK_WINDOW (dialog), parent);
 	}
 
-	if (default_module)
-		gtk_widget_grab_focus (modules_view);
-	else
-		gtk_widget_grab_focus (targets_view);
+	gtk_widget_grab_focus (modules_view);
 
 	/* execute dialog */
 	while (!finished)
@@ -1582,10 +1672,11 @@ anjuta_pm_project_new_module (ProjectManagerPlugin *plugin,
 			}
 			case GTK_RESPONSE_OK:
 			{
+				GFile *target_file;
 				AnjutaProjectNode *target;
 
-				target = gbf_project_view_find_selected (GBF_PROJECT_VIEW (targets_view),
-														ANJUTA_PROJECT_TARGET);
+				target_file = ianjuta_project_chooser_get_selected (IANJUTA_PROJECT_CHOOSER (target_chooser), NULL);
+				target = gbf_project_view_get_node_from_file (plugin->view, ANJUTA_PROJECT_UNKNOWN, target_file);
 				if (target)
 				{
 					GString *err_mesg = g_string_new (NULL);
@@ -1905,6 +1996,9 @@ anjuta_pm_project_new_package (ProjectManagerPlugin *plugin,
 				g_string_free (error_message, TRUE);
                 break;
             }
+			case GTK_RESPONSE_HELP:
+				anjuta_util_help_display (GTK_WIDGET (dialog), ANJUTA_MANUAL, ADD_PACKAGE_HELP);
+				break;
             default:
                 finished = TRUE;
                 break;
