@@ -65,38 +65,48 @@
 #define NEW_PROJECT_DIALOG "druid_window"
 #define PROJECT_LIST "project_list"
 #define PROJECT_BOOK "project_book"
+#define PROJECT_PAGE "project_page"
+#define ERROR_PAGE "error_page"
+#define ERROR_TITLE "error_title"
+#define PROGRESS_PAGE "progress_page"
+#define FINISH_PAGE "finish_page"
+#define FINISH_TEXT "finish_text"
+#define PROPERTY_PAGE "property_page"
+#define PROPERTY_TITLE "property_title"
 #define PROPERTY_TABLE "property_table"
 #define ERROR_VBOX "error_vbox"
 #define ERROR_ICON "error_icon"
 #define ERROR_MESSAGE "error_message"
 #define ERROR_DETAIL "error_detail"
 
-#define PROJECT_PAGE 0
-#define ERROR_PAGE 1
-#define PROGRESS_PAGE 2
-#define FINISH_PAGE 3
-#define PROPERTY_PAGE 4
+
+#define PROJECT_PAGE_INDEX 0
 
 /*---------------------------------------------------------------------------*/
 
 struct _NPWDruid
 {
 	GtkWindow* window;
-	
+
 	GtkNotebook* project_book;
+	GtkWidget *error_page;
+	GtkWidget *error_title;
 	GtkVBox *error_vbox;
 	GtkWidget *error_extra_widget;
 	GtkImage *error_icon;
 	GtkLabel *error_message;
 	GtkWidget *error_detail;
 
+	GtkWidget *project_page;
+	GtkWidget *progress_page;
+	GtkWidget *finish_page;
+	GtkWidget *finish_text;
+
 	const gchar* project_file;
 	NPWPlugin* plugin;
-	
-	gint next_page;
-	gint last_page;
-	
+
 	GQueue* page_list;
+
 	GHashTable* values;
 	NPWPageParser* parser;
 	GList* header_list;
@@ -113,6 +123,9 @@ enum {
 	DATA_COLUMN
 };
 
+/* Helper functon
+ *---------------------------------------------------------------------------*/
+
 /* Create error page
  *---------------------------------------------------------------------------*/
 
@@ -126,10 +139,13 @@ npw_druid_fill_error_page (NPWDruid* druid, GtkWidget *extra_widget, GtkMessageT
 	gchar *message;
 	const gchar *stock_id = NULL;
 	const gchar *title = NULL;
-	
+
 	assistant = GTK_ASSISTANT (druid->window);
-	page = gtk_assistant_get_nth_page (assistant, ERROR_PAGE);
-	
+
+	/* Add page to assistant after current one */
+	page = druid->error_page;
+	gtk_assistant_insert_page (assistant, page, gtk_assistant_get_current_page (assistant) + 1);
+
 	/* Set dialog kind */
 	switch (type)
 	{
@@ -155,7 +171,8 @@ npw_druid_fill_error_page (NPWDruid* druid, GtkWidget *extra_widget, GtkMessageT
 	default:
 		g_warning ("Unknown GtkMessageType %u", type);
 		break;
-	}	
+	}
+	gtk_label_set_text (GTK_LABEL (druid->error_title), title);
 	gtk_assistant_set_page_title (assistant, page, title);
 	if (type == GTK_MESSAGE_ERROR)
 	{
@@ -168,14 +185,14 @@ npw_druid_fill_error_page (NPWDruid* druid, GtkWidget *extra_widget, GtkMessageT
 		gtk_assistant_set_page_complete (assistant, page, TRUE);
 	}
 	gtk_image_set_from_stock (druid->error_icon, stock_id, GTK_ICON_SIZE_DIALOG);
-	
+
 	/* Set message */
 	va_start (args, mesg);
 	message = g_strdup_vprintf (mesg, args);
 	va_end (args);
 	gtk_label_set_markup (druid->error_message, message);
 	g_free (message);
-	
+
 	/* Set detail */
 	if (detail == NULL)
 	{
@@ -184,7 +201,7 @@ npw_druid_fill_error_page (NPWDruid* druid, GtkWidget *extra_widget, GtkMessageT
 	else
 	{
 		GtkLabel *label;
-		
+
 		gtk_widget_show (druid->error_detail);
 		label = GTK_LABEL (gtk_bin_get_child (GTK_BIN (druid->error_detail)));
 		gtk_label_set_text (label, detail);
@@ -215,7 +232,7 @@ cb_druid_add_summary_property (NPWProperty* property, gpointer user_data)
 	if (npw_property_get_options (property) & NPW_SUMMARY_OPTION)
 	{
 		g_string_append_printf (text, "%s %s\n",
-								npw_property_get_label (property), 
+								npw_property_get_label (property),
 								npw_property_get_value (property));
 	}
 }
@@ -240,12 +257,12 @@ npw_druid_fill_summary_page (NPWDruid* druid)
 	{
 		npw_page_foreach_property (page, (GFunc)cb_druid_add_summary_property, text);
 	}
-	
-	label = GTK_LABEL (gtk_assistant_get_nth_page (GTK_ASSISTANT (druid->window), FINISH_PAGE));
+
+	label = GTK_LABEL (druid->finish_text);
 	gtk_label_set_markup (label, text->str);
 	g_string_free (text, TRUE);
-	
-	gtk_assistant_set_page_complete (GTK_ASSISTANT (druid->window), GTK_WIDGET (label), TRUE);
+
+	gtk_assistant_set_page_complete (GTK_ASSISTANT (druid->window), druid->finish_page, TRUE);
 }
 
 /* Create project selection page
@@ -257,14 +274,14 @@ on_druid_project_update_selected (GtkIconView* view, NPWDruid *druid)
 {
 	GList *selected;
 	NPWHeader* header = NULL;
-	
+
 	/* No item can be selected when the view is mapped */
 	selected = gtk_icon_view_get_selected_items (view);
 	if (selected != NULL)
 	{
 		GtkTreeIter iter;
 		GtkTreeModel *model;
-	
+
 		model = gtk_icon_view_get_model (view);
 		if (gtk_tree_model_get_iter (model, &iter, (GtkTreePath *)selected->data))
 		{
@@ -275,9 +292,7 @@ on_druid_project_update_selected (GtkIconView* view, NPWDruid *druid)
 	}
 
 	druid->header = header;
-	gtk_assistant_set_page_complete (GTK_ASSISTANT (druid->window),
-									 gtk_assistant_get_nth_page (GTK_ASSISTANT (druid->window),PROJECT_PAGE),
-									 header != NULL);
+	gtk_assistant_set_page_complete (GTK_ASSISTANT (druid->window), druid->project_page, header != NULL);
 }
 
 /* Add a project in the icon list */
@@ -296,7 +311,7 @@ cb_druid_insert_project_icon (gpointer data, gpointer user_data)
 						DESC_COLUMN, npw_header_get_description (header),
 						DATA_COLUMN, header,
 						-1);
-	
+
 	g_object_unref (pixbuf);
 }
 
@@ -316,43 +331,43 @@ cb_druid_insert_project_page (gpointer value, gpointer user_data)
 	const gchar* category;
 
 	category = npw_header_get_category ((NPWHeader *)template_list->data);
-	
-	/* Build another complete wizard dialog from the gtk builder file 
+
+	/* Build another complete wizard dialog from the gtk builder file
 	 * but keep only the project selection notebook page */
 	builder = gtk_builder_new ();
 	if (!gtk_builder_add_from_file (builder, GTK_BUILDER_UI_FILE, NULL))
 	{
 		g_warn_if_reached ();
-		g_object_unref (builder); 
+		g_object_unref (builder);
 		return;
 	}
-	
+
 	/* Fill icon view */
 	view = GTK_ICON_VIEW (gtk_builder_get_object (builder, PROJECT_LIST));
 	gtk_icon_view_set_pixbuf_column (view, PIXBUF_COLUMN);
 	gtk_icon_view_set_markup_column (view, TEXT_COLUMN);
 	store = gtk_list_store_new (4, GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER);
-	g_list_foreach (template_list, cb_druid_insert_project_icon, store); 
+	g_list_foreach (template_list, cb_druid_insert_project_icon, store);
 	gtk_icon_view_set_model (view, GTK_TREE_MODEL (store));
-	
+
 	/* Connect signal to update dialog */
 	g_signal_connect (G_OBJECT (view), "selection-changed", G_CALLBACK (on_druid_project_update_selected), druid);
 	g_signal_connect (G_OBJECT (view), "map", G_CALLBACK (on_druid_project_update_selected), druid);
 
 	/* Set new page label */
 	assistant = GTK_WIDGET (gtk_builder_get_object (builder, NEW_PROJECT_DIALOG));
-	book = GTK_NOTEBOOK (gtk_assistant_get_nth_page (GTK_ASSISTANT (assistant), PROJECT_PAGE));
+	book = GTK_NOTEBOOK (gtk_builder_get_object (builder, PROJECT_BOOK));
 	child = gtk_notebook_get_nth_page (book, 0);
 	label = gtk_notebook_get_tab_label (book, child);
-	gtk_label_set_text (GTK_LABEL(label), (const gchar *)category);	
-	
+	gtk_label_set_text (GTK_LABEL(label), (const gchar *)category);
+
 	/* Pick up the filled project selection page from the newly created dialog
-	 * add it to the wizard project notebook and destroy the dialog 
+	 * add it to the wizard project notebook and destroy the dialog
 	 */
 	gtk_notebook_remove_page (book, 0);
 	gtk_notebook_append_page (druid->project_book, child, label);
 	gtk_widget_destroy (assistant);
-	
+
 	g_object_unref (builder);
 }
 
@@ -362,15 +377,14 @@ npw_druid_fill_selection_page (NPWDruid* druid, const gchar *directory)
 {
 	gchar* dir;
 	const gchar * const * sys_dir;
- 
+
 	/* Remove all previous data */
-	druid->project_book = GTK_NOTEBOOK (gtk_assistant_get_nth_page (GTK_ASSISTANT (druid->window), PROJECT_PAGE));
-	gtk_notebook_remove_page(druid->project_book, 0);	
-	npw_header_list_free (druid->header_list);	
-	
+	gtk_notebook_remove_page(druid->project_book, 0);
+	npw_header_list_free (druid->header_list);
+
 	/* Create list of projects */
-	druid->header_list = npw_header_list_new ();	
-	
+	druid->header_list = npw_header_list_new ();
+
 	if (directory != NULL)
 	{
 		/* Read project template only in specified directory */
@@ -384,8 +398,8 @@ npw_druid_fill_selection_page (NPWDruid* druid, const gchar *directory)
 		dir = g_build_filename (g_get_user_data_dir (), "anjuta", "project", NULL);
 		npw_header_list_readdir (&druid->header_list, dir);
 		g_free (dir);
-	
-		/* Read project template in system directory */	
+
+		/* Read project template in system directory */
 		for (sys_dir = g_get_system_data_dirs (); *sys_dir != NULL; sys_dir++)
 		{
 			dir = g_build_filename (*sys_dir, "anjuta", "project", NULL);
@@ -396,10 +410,10 @@ npw_druid_fill_selection_page (NPWDruid* druid, const gchar *directory)
 		/* Read anjuta installation directory */
 		npw_header_list_readdir (&druid->header_list, PROJECT_WIZARD_DIRECTORY);
 	}
-	
+
 	if (g_list_length (druid->header_list) == 0)
 	{
-		anjuta_util_dialog_error (GTK_WINDOW (ANJUTA_PLUGIN (druid->plugin)->shell),_("Unable to find any project template in %s"), PROJECT_WIZARD_DIRECTORY);		
+		anjuta_util_dialog_error (GTK_WINDOW (ANJUTA_PLUGIN (druid->plugin)->shell),_("Unable to find any project template in %s"), PROJECT_WIZARD_DIRECTORY);
 		return FALSE;
 	}
 
@@ -407,7 +421,7 @@ npw_druid_fill_selection_page (NPWDruid* druid, const gchar *directory)
 	g_list_foreach (druid->header_list, cb_druid_insert_project_page, druid);
 
 	gtk_widget_show_all (GTK_WIDGET (druid->project_book));
-	
+
 	return TRUE;
 }
 
@@ -444,7 +458,7 @@ cb_druid_add_property (NPWProperty* property, gpointer user_data)
 	{
 		/* Not hidden property */
 		description = npw_property_get_description (property);
-	
+
 		/* Set description tooltip */
 		if (description && (*description != '\0'))
 		{
@@ -454,9 +468,9 @@ cb_druid_add_property (NPWProperty* property, gpointer user_data)
 		label = gtk_label_new (npw_property_get_label (property));
 		gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
 		gtk_misc_set_padding (GTK_MISC (label), 6, 6);
-		
+
 		switch (npw_property_get_type (property))
-		{				
+		{
 			case NPW_PACKAGE_PROPERTY:
 				gtk_table_resize (data->table, data->row + 2, 1);
 				gtk_table_attach (data->table, label, 0, 1, data->row, data->row + 1,
@@ -478,7 +492,7 @@ cb_druid_add_property (NPWProperty* property, gpointer user_data)
 				                  (GtkAttachOptions)(attach), 0, 0, 0);
 				data->row++;
 		}
-		
+
 		/* Set first entry */
 		if (data->first_entry == NULL) data->first_entry = entry;
 	}
@@ -488,14 +502,21 @@ static void
 npw_druid_fill_property_page (NPWDruid* druid, NPWPage* page)
 {
 	GtkWidget *widget;
+	GList *children;
+	GtkLabel *label;
 	NPWDruidAddPropertyData data;
 
-	widget = gtk_assistant_get_nth_page (GTK_ASSISTANT (druid->window), druid->next_page);
+	/* Add page to assistant, after current page */
+	widget = gtk_assistant_get_nth_page (GTK_ASSISTANT (druid->window), gtk_assistant_get_current_page (GTK_ASSISTANT (druid->window)) + 1);
 
 	/* Remove previous widgets */
 	gtk_container_foreach (GTK_CONTAINER (npw_page_get_widget (page)), cb_druid_destroy_widget, NULL);
- 		
+
 	/* Update title	*/
+	children = gtk_container_get_children (GTK_CONTAINER (widget));
+	label = GTK_LABEL (g_list_nth_data (children, 0));
+	g_list_free (children);
+	gtk_label_set_text (label, npw_page_get_label (page));
 	gtk_assistant_set_page_title (GTK_ASSISTANT (druid->window), widget, npw_page_get_label (page));
 
 	/* Add new widget */
@@ -510,7 +531,7 @@ npw_druid_fill_property_page (NPWDruid* druid, NPWPage* page)
 	{
 		gtk_container_set_focus_child (GTK_CONTAINER (data.table), data.first_entry);
 	}
-	
+
 	gtk_widget_show_all (widget);
 }
 
@@ -520,10 +541,13 @@ npw_druid_fill_property_page (NPWDruid* druid, NPWPage* page)
 static NPWPage*
 npw_druid_add_new_page (NPWDruid* druid)
 {
+	gint current;
 	NPWPage* page;
 
 	/* Get page in cache */
-	page = g_queue_peek_nth (druid->page_list, druid->next_page - PROPERTY_PAGE);
+	current = gtk_assistant_get_current_page (GTK_ASSISTANT (druid->window));
+	page = g_queue_peek_nth (druid->page_list, current);
+
 	if (page == NULL)
 	{
 		/* Page not found in cache, create */
@@ -534,37 +558,37 @@ npw_druid_add_new_page (NPWDruid* druid)
 		GtkWidget *table;
 		GtkAssistant *assistant;
 
-		/* Build another complete wizard dialog from the gtk builder file 
+		/* Build another complete wizard dialog from the gtk builder file
 	 	* but keep only the property assistant page */
 		builder = gtk_builder_new ();
 		if (!gtk_builder_add_from_file (builder, GTK_BUILDER_UI_FILE, NULL))
 		{
 			g_warn_if_reached ();
-			g_object_unref (builder); 
-			
+			g_object_unref (builder);
+
 			return NULL;
 		}
 		assistant = GTK_ASSISTANT (gtk_builder_get_object (builder, NEW_PROJECT_DIALOG));
+		widget = GTK_WIDGET (gtk_builder_get_object (builder, PROPERTY_PAGE));
 		table = GTK_WIDGET (gtk_builder_get_object (builder, PROPERTY_TABLE));
-		
-		widget = gtk_assistant_get_nth_page (assistant, PROPERTY_PAGE);
+
 		type = gtk_assistant_get_page_type (assistant, widget);
 		pixbuf = gtk_assistant_get_page_header_image (assistant, widget);
 		if (pixbuf) g_object_ref (pixbuf);
 		gtk_container_remove (GTK_CONTAINER (assistant), widget);
-		gtk_assistant_insert_page (GTK_ASSISTANT (druid->window), widget, druid->next_page);
+		gtk_assistant_insert_page (GTK_ASSISTANT (druid->window), widget, current + 1);
 		gtk_assistant_set_page_type (GTK_ASSISTANT (druid->window), widget, type);
 		if (pixbuf != NULL)
-		{	
+		{
 			gtk_assistant_set_page_header_image (GTK_ASSISTANT (druid->window), widget, pixbuf);
 			g_object_ref (pixbuf);
 		}
-		gtk_assistant_set_page_complete (GTK_ASSISTANT (druid->window), widget, TRUE);		
+		gtk_assistant_set_page_complete (GTK_ASSISTANT (druid->window), widget, TRUE);
 		gtk_widget_destroy (GTK_WIDGET (assistant));
-		
-		/* Builder get reference on all built widget, so unref it when all work is done */		
+
+		/* Builder get reference on all built widget, so unref it when all work is done */
 		g_object_unref (builder);
-		
+
 		/* Create new page */
 		page = npw_page_new (druid->values);
 		npw_page_set_widget (page, table);
@@ -576,25 +600,26 @@ npw_druid_add_new_page (NPWDruid* druid)
 	return page;
 }
 
-/* Remove all following page */
+/* Remove all pages following current page except the summary page*/
 
 static void
 npw_druid_remove_following_page (NPWDruid* druid)
 {
-	NPWPage* page;
-	gint num = druid->next_page;
+	gint current;
 
+	current = gtk_assistant_get_current_page (GTK_ASSISTANT (druid->window));
 	for(;;)
 	{
 		GtkWidget *widget;
-		
-		page = g_queue_pop_nth (druid->page_list, num - PROPERTY_PAGE);
-		if (page == NULL) break;
-		
-		widget = gtk_assistant_get_nth_page (GTK_ASSISTANT (druid->window), num);
+		NPWPage* page;
+
+		widget = gtk_assistant_get_nth_page (GTK_ASSISTANT (druid->window), current + 1);
+		if (widget == druid->finish_page) break;
+
 		gtk_container_remove (GTK_CONTAINER (druid->window), widget);
-		
-		npw_page_free (page);
+
+		page = g_queue_pop_nth (druid->page_list, current - 1);
+		if (page != NULL) npw_page_free (page);
 	}
 }
 
@@ -607,7 +632,7 @@ typedef struct _NPWSaveValidPropertyData
 	gboolean modified;
 	GString *error;
 	GString *warning;
-	
+
 } NPWSaveValidPropertyData;
 
 static void
@@ -621,8 +646,8 @@ cb_save_valid_property (NPWProperty* property, gpointer user_data)
 	modified = npw_property_update_value_from_widget (property);
 	if (modified) data->modified = modified;
        	value = npw_property_get_value (property);
-	
-	/* Check mandatory property */ 
+
+	/* Check mandatory property */
 	if (modified &&  (npw_property_get_options (property) & NPW_MANDATORY_OPTION))
 	{
 		if ((value == NULL) || (strlen (value) <= 0))
@@ -638,7 +663,7 @@ cb_save_valid_property (NPWProperty* property, gpointer user_data)
 	if (modified && !npw_property_is_valid_restriction (property))
 	{
 		NPWPropertyRestriction restriction = npw_property_get_restriction (property);
-		
+
 		switch (restriction)
 		{
 		case NPW_FILENAME_RESTRICTION:
@@ -651,10 +676,15 @@ cb_save_valid_property (NPWProperty* property, gpointer user_data)
 									_("Field \"%s\" must contains only letters, digits, the following characters \"#$:%%+,.=@^_`~\" or directory separators. In addition you cannot have a leading dash. Please fix it."),
 									npw_property_get_label (property));
 			break;
+		case NPW_PRINTABLE_RESTRICTION:
+			g_string_append_printf (data->error,
+									_("Field \"%s\" must contains only ASCII printable characters, no accentuated characters by example. Please fix it."),
+									npw_property_get_label (property));
+			break;
 		default:
 			g_string_append_printf (data->error,
 									_("Unknown error."));
-									
+
 		}
 		npw_property_remove_value (property);
 	}
@@ -679,7 +709,7 @@ cb_save_valid_property (NPWProperty* property, gpointer user_data)
 
 		if (exist)
 		{
-			g_string_append_printf (data->warning, is_directory ? 
+			g_string_append_printf (data->warning, is_directory ?
 										_("Directory \"%s\" is not empty. Project creation could fail if some files "
 										  "cannot be written. Do you want to continue?") :
 											_("File \"%s\" already exists. Do you want to overwrite it?"),
@@ -694,11 +724,13 @@ cb_save_valid_property (NPWProperty* property, gpointer user_data)
 static gboolean
 npw_druid_save_valid_values (NPWDruid* druid)
 {
+	gint current;
 	NPWPage* page;
 	NPWSaveValidPropertyData data;
 	gboolean ok = TRUE;
 
-	page = g_queue_peek_nth (druid->page_list, druid->next_page - PROPERTY_PAGE - 1);
+	current = gtk_assistant_get_current_page (GTK_ASSISTANT (druid->window)) - 2;
+	page = g_queue_peek_nth (druid->page_list, current);
 	data.modified = FALSE;
 	data.parent = GTK_WINDOW (druid->window);
 	data.error = g_string_new (NULL);
@@ -706,7 +738,7 @@ npw_druid_save_valid_values (NPWDruid* druid)
 	npw_page_foreach_property (page, (GFunc)cb_save_valid_property, &data);
 
 	if (data.modified) npw_druid_remove_following_page (druid);
-	
+
 	if (data.error->len)
 	{
 		npw_druid_fill_error_page (druid, NULL,
@@ -715,7 +747,6 @@ npw_druid_save_valid_values (NPWDruid* druid)
 									"<b>%s</b>\n\n%s",
 									_("Invalid entry"),
 									data.error->str);
-		gtk_assistant_set_current_page (GTK_ASSISTANT (druid->window), ERROR_PAGE);
 		ok = FALSE;
 	}
 	else if (data.warning->len)
@@ -726,10 +757,9 @@ npw_druid_save_valid_values (NPWDruid* druid)
 									"<b>%s</b>\n\n%s",
 									_("Dubious entry"),
 									data.warning->str);
-		gtk_assistant_set_current_page (GTK_ASSISTANT (druid->window), ERROR_PAGE);
 		ok = FALSE;
 	}
-	
+
 	g_string_free (data.error, TRUE);
 	g_string_free (data.warning, TRUE);
 
@@ -744,7 +774,7 @@ npw_druid_set_busy (NPWDruid *druid, gboolean busy_state)
 {
 	if (druid->busy == busy_state)
 		return;
-	
+
 	/* Set busy state */
 	if (busy_state)
 		anjuta_status_busy_push (anjuta_shell_get_status (ANJUTA_PLUGIN (druid->plugin)->shell, NULL));
@@ -786,23 +816,28 @@ static void
 on_druid_get_new_page (NPWAutogen* gen, gpointer data)
 {
 	NPWDruid* druid = (NPWDruid *)data;
+	gint current;
 	NPWPage* page;
 
-	page = g_queue_peek_nth (druid->page_list, druid->next_page - PROPERTY_PAGE);
-	
+	current = gtk_assistant_get_current_page (GTK_ASSISTANT (druid->window));
+	page = g_queue_peek_nth (druid->page_list, current - 1);
+
 	if (npw_page_get_name (page) == NULL)
 	{
 		/* no page, display finish page */
 		npw_druid_fill_summary_page (druid);
-		
-		gtk_assistant_set_current_page (GTK_ASSISTANT (druid->window), FINISH_PAGE);
+
+		page = g_queue_pop_nth (druid->page_list, current - 1);
+		if (page != NULL) npw_page_free (page);
+		gtk_container_remove (GTK_CONTAINER (druid->window), gtk_assistant_get_nth_page (GTK_ASSISTANT (druid->window), current + 1));
+		gtk_assistant_set_current_page (GTK_ASSISTANT (druid->window), current + 1);
 	}
 	else
 	{
 		/* display property page */
 		npw_druid_fill_property_page (druid, page);
 
-		gtk_assistant_set_current_page (GTK_ASSISTANT (druid->window), druid->next_page);
+		gtk_assistant_set_current_page (GTK_ASSISTANT (druid->window), current + 1);
 	}
 }
 
@@ -811,7 +846,7 @@ on_druid_parse_page (const gchar* output, gpointer data)
 {
 	GError *error = NULL;
 	NPWPageParser* parser = (NPWPageParser*)data;
-	
+
 	npw_page_parser_parse (parser, output, strlen (output), &error);
 
 	if (error)
@@ -837,7 +872,7 @@ on_install_button_clicked (GtkWidget *button, NPWDruid *druid)
 	GList *missing_programs, *missing_packages;
 	GList *missing_files = NULL;
 	GList *node;
-	
+
 
 	missing_programs = npw_header_check_required_programs (druid->header);
 	missing_packages = npw_header_check_required_packages (druid->header);
@@ -855,7 +890,7 @@ on_install_button_clicked (GtkWidget *button, NPWDruid *druid)
 		missing_files = g_list_append (missing_files, pk_pkg_config_string);
 	}
 	g_list_free (missing_packages);
-	
+
 	if (missing_files)
 	{
 		gchar * missing_names = NULL;
@@ -882,7 +917,7 @@ check_and_warn_missing (NPWDruid *druid)
 	{
 		missing_message = g_string_new (NULL);
 	}
-	
+
 	if (missing_programs)
 	{
 		gchar *missing_progs;
@@ -893,7 +928,7 @@ check_and_warn_missing (NPWDruid *druid)
 		g_free (missing_progs);
 		anjuta_util_glist_strings_free (missing_programs);
 	}
-	
+
 	if (missing_packages)
 	{
 		gchar *missing_pkgs;
@@ -904,7 +939,7 @@ check_and_warn_missing (NPWDruid *druid)
 		g_free (missing_pkgs);
 		anjuta_util_glist_strings_free (missing_packages);
 	}
-	
+
 	if (missing_message)
 	{
 		GtkWidget *hbox, *install_button;
@@ -916,12 +951,14 @@ check_and_warn_missing (NPWDruid *druid)
 		hbox = gtk_hbox_new (FALSE, 0);
 		gtk_widget_show (hbox);
 
+#ifdef ENABLE_PACKAGEKIT
 		install_button =
 			gtk_button_new_with_label (_("Install missing packages"));
 		gtk_box_pack_end (GTK_BOX (hbox), install_button, FALSE, FALSE, 10);
 		g_signal_connect (install_button, "clicked",
 						  G_CALLBACK (on_install_button_clicked), druid);
 		gtk_widget_show (install_button);
+#endif
 
 		npw_druid_fill_error_page (druid, hbox,
 								   GTK_MESSAGE_WARNING,
@@ -939,62 +976,51 @@ check_and_warn_missing (NPWDruid *druid)
 									_("Missing components"),
 									missing_message->str);
 		g_string_free (missing_message, TRUE);
-		
-		gtk_assistant_set_current_page (GTK_ASSISTANT (druid->window), ERROR_PAGE);
 	}
-	
+
 	return !missing_message;
 }
 
-/* This function is called to compute the next page AND to determine the status
- * of last button: it does not mean that the user go to the next page. 
- * We need this information to generate the next page just before displaying it.
- * So a progress page is inserted between all pages of the wizard. We use the
- * prepare signal handler of this page to compute the next page and switch to
- * it using gtk_assistant_set_current_page(). */
-static gint
-on_druid_next (gint page, gpointer user_data)
-{
-	return page == FINISH_PAGE ? -1 : PROGRESS_PAGE;
-}
 
-static gboolean
-on_druid_delayed_get_new_page (gpointer data)
-{
-	on_druid_get_new_page (NULL, data);
-
-	return FALSE;
-}
-	
 static void
-on_druid_prepare (GtkAssistant* assistant, GtkWidget *page, NPWDruid* druid)
+on_druid_real_prepare (GtkAssistant* assistant, GtkWidget *page, NPWDruid* druid)
 {
-	gint current_page = gtk_assistant_get_current_page (assistant);
-	
-	if (current_page == PROGRESS_PAGE)
+	if (page == druid->progress_page)
 	{
-		/* Generate the next page */
+		gint previous;
+		gboolean last_warning;
 
-		if (druid->next_page == PROPERTY_PAGE)
+		previous = gtk_assistant_get_current_page (assistant) - 1;
+		last_warning = gtk_assistant_get_nth_page (assistant, previous) == druid->error_page;
+		if (last_warning)
+		{
+			/* Remove warning page */
+			gtk_container_remove (GTK_CONTAINER (assistant), druid->error_page);
+			previous--;
+		}
+
+		/* Generate the next page */
+		if (previous == PROJECT_PAGE_INDEX)
 		{
 			const gchar* new_project;
 
 			new_project = npw_header_get_filename (druid->header);
-			
+
 			if (druid->project_file != new_project)
-			{	
-				if (druid->last_page != ERROR_PAGE)
+			{
+				npw_druid_remove_following_page (druid);
+
 				/* Check if necessary programs for this project is installed */
-				if (!check_and_warn_missing (druid))
-				{   
+				if (!last_warning && !check_and_warn_missing (druid))
+				{
+					gtk_assistant_set_current_page (assistant, gtk_assistant_get_current_page (assistant) + 1);
 					return;
-				}   
-				
+				}
+
 				/* Change project */
 				druid->project_file = new_project;
-				npw_druid_remove_following_page (druid);
 				npw_autogen_set_input_file (druid->gen, druid->project_file, "[+","+]");
-				
+
 			}
 		}
 		else
@@ -1002,20 +1028,20 @@ on_druid_prepare (GtkAssistant* assistant, GtkWidget *page, NPWDruid* druid)
 			if (!npw_druid_save_valid_values (druid))
 			{
 				/* Display error */
-				gtk_assistant_set_current_page (assistant, ERROR_PAGE);
-				
+				gtk_assistant_set_current_page (assistant, gtk_assistant_get_current_page (assistant) + 1);
+
 				return;
 			}
 		}
 
-		if (g_queue_peek_nth (druid->page_list, druid->next_page - PROPERTY_PAGE) == NULL)
+		if (g_queue_peek_nth (druid->page_list, previous) == NULL)
 		{
 			/* Regenerate new page */
 			gtk_assistant_set_page_complete (assistant, page, FALSE);
 			if (druid->parser != NULL)
 				npw_page_parser_free (druid->parser);
-			druid->parser = npw_page_parser_new (npw_druid_add_new_page (druid), druid->project_file, druid->next_page - PROPERTY_PAGE);
-			
+			druid->parser = npw_page_parser_new (npw_druid_add_new_page (druid), druid->project_file, previous);
+
 			npw_autogen_set_output_callback (druid->gen, on_druid_parse_page, druid->parser);
 			npw_autogen_write_definition_file (druid->gen, druid->values);
 			npw_autogen_execute (druid->gen, on_druid_get_new_page, druid, NULL);
@@ -1023,35 +1049,53 @@ on_druid_prepare (GtkAssistant* assistant, GtkWidget *page, NPWDruid* druid)
 		else
 		{
 			/* Page is already in cache, change the page to display it */
-			/* The page change is delayed because in the latest version of
-			 * GtkAssistant, the page switch is not completely done when
-			 * the signal is called. A page change in the signal handler
-			 * will be partialy overwritten */
-			g_idle_add (on_druid_delayed_get_new_page, druid);
+			on_druid_get_new_page (NULL, druid);
 		}
 	}
-	else if (current_page == ERROR_PAGE)
-	{
-		druid->last_page = ERROR_PAGE;
-	}
-	else if (current_page == PROJECT_PAGE)
-	{
-		druid->last_page = current_page;
-		druid->next_page = PROPERTY_PAGE;
-	}
-	else if (current_page == FINISH_PAGE)
+	else if (page == druid->finish_page)
 	{
 		npw_druid_set_busy (druid, FALSE);
+		gtk_container_remove (GTK_CONTAINER (assistant), druid->error_page);
+		gtk_container_remove (GTK_CONTAINER (assistant), druid->progress_page);
 	}
-	else if (current_page >= PROPERTY_PAGE)
+	else
 	{
 		npw_druid_set_busy (druid, FALSE);
-		druid->last_page = current_page;
-		druid->next_page = current_page + 1;
+
+		if (page != druid->error_page) gtk_container_remove (GTK_CONTAINER (assistant), druid->error_page);
+
+		/* Move progress page */
+		gtk_container_remove (GTK_CONTAINER (assistant), druid->progress_page);
+		gtk_assistant_insert_page (assistant, druid->progress_page, gtk_assistant_get_current_page (assistant) + 1);
+		gtk_assistant_set_page_title (assistant, druid->progress_page, "...");
 	}
-	
 }
 
+static gboolean
+on_druid_delayed_prepare (gpointer data)
+{
+	NPWDruid *druid = (NPWDruid *)data;
+	GtkAssistant *assistant;
+	GtkWidget *page;
+
+	assistant = GTK_ASSISTANT (druid->window);
+	page = gtk_assistant_get_nth_page (assistant, gtk_assistant_get_current_page (assistant));
+	on_druid_real_prepare (assistant, page, druid);
+
+	return FALSE;
+}
+
+static void
+on_druid_prepare (GtkAssistant* assistant, GtkWidget *page, NPWDruid* druid)
+{
+	/* The page change is delayed because in the latest version of
+	 * GtkAssistant, the page switch is not completely done when
+	 * the signal is called. A page change in the signal handler
+	 * will be partialy overwritten */
+	g_idle_add (on_druid_delayed_prepare, druid);
+}
+
+
 static void
 on_druid_finish (GtkAssistant* assistant, NPWDruid* druid)
 {
@@ -1068,12 +1112,12 @@ npw_druid_create_assistant (NPWDruid* druid, const gchar *directory)
 {
 	AnjutaShell *shell;
 	GtkBuilder *builder;
-	GError* error = NULL;		
+	GError* error = NULL;
 	GtkAssistant *assistant;
-	GtkWidget *page;
-	
+	GtkWidget *property_page;
+
 	g_return_val_if_fail (druid->window == NULL, NULL);
-	
+
 	shell = ANJUTA_PLUGIN (druid->plugin)->shell;
 
 	/* Create GtkAssistant using GtkBuilder, glade doesn't seem to work*/
@@ -1084,46 +1128,51 @@ npw_druid_create_assistant (NPWDruid* druid, const gchar *directory)
 		g_error_free (error);
 		return NULL;
 	}
-	assistant = GTK_ASSISTANT (gtk_builder_get_object (builder, NEW_PROJECT_DIALOG));
+	anjuta_util_builder_get_objects (builder,
+	                                 NEW_PROJECT_DIALOG, &assistant,
+	                                 PROJECT_BOOK, &druid->project_book,
+	                                 ERROR_VBOX, &druid->error_vbox,
+	                                 ERROR_TITLE, &druid->error_title,
+	                                 ERROR_ICON, &druid->error_icon,
+	                                 ERROR_MESSAGE, &druid->error_message,
+	                                 ERROR_DETAIL, &druid->error_detail,
+	                                 PROJECT_PAGE, &druid->project_page,
+	                                 ERROR_PAGE, &druid->error_page,
+	                                 PROGRESS_PAGE, &druid->progress_page,
+	                                 FINISH_PAGE, &druid->finish_page,
+	                                 FINISH_TEXT, &druid->finish_text,
+	                                 PROPERTY_PAGE, &property_page,
+	                                 NULL);
 	druid->window = GTK_WINDOW (assistant);
-	druid->project_book = GTK_NOTEBOOK (gtk_builder_get_object (builder, PROJECT_BOOK));
-	druid->error_vbox = GTK_VBOX (gtk_builder_get_object (builder, ERROR_VBOX));
-	druid->error_icon = GTK_IMAGE (gtk_builder_get_object (builder, ERROR_ICON));
-	druid->error_message = GTK_LABEL (gtk_builder_get_object (builder, ERROR_MESSAGE));
-	druid->error_detail = GTK_WIDGET (gtk_builder_get_object (builder, ERROR_DETAIL));
+	g_object_ref (druid->error_page);
+	g_object_ref (druid->progress_page);
 	gtk_window_set_transient_for (GTK_WINDOW (assistant), GTK_WINDOW (shell));
 	g_object_unref (builder);
 
 	/* Connect assistant signals */
-	gtk_assistant_set_forward_page_func (assistant, on_druid_next, druid, NULL);
 	g_signal_connect (G_OBJECT (assistant), "prepare", G_CALLBACK (on_druid_prepare), druid);
 	g_signal_connect (G_OBJECT (assistant), "apply", G_CALLBACK (on_druid_finish), druid);
 	g_signal_connect (G_OBJECT (assistant), "cancel", G_CALLBACK (on_druid_cancel), druid);
 	g_signal_connect (G_OBJECT (assistant), "close", G_CALLBACK (on_druid_close), druid);
 	g_signal_connect(G_OBJECT(assistant), "key-press-event", G_CALLBACK(on_project_wizard_key_press_event), druid);
 
+	/* Remove property page, will be created later as needed */
+	gtk_container_remove (GTK_CONTAINER (assistant), property_page);
+
 	/* Setup project selection page */
 	if (!npw_druid_fill_selection_page (druid, directory))
 	{
 		return NULL;
 	}
 
-	/* Use progress page to stop the flow */
-	/*page = gtk_assistant_get_nth_page (assistant, PROGRESS_PAGE);
-	gtk_assistant_set_page_complete (assistant, page, FALSE);*/
-	
-	/* Remove property page, will be created later as needed */
-	page = gtk_assistant_get_nth_page (assistant, PROPERTY_PAGE);
-	gtk_container_remove (GTK_CONTAINER (assistant), page);
-	
 	/* Add dialog widget to anjuta status. */
 	anjuta_status_add_widget (anjuta_shell_get_status (shell, NULL), GTK_WIDGET (assistant));
 
 	gtk_window_set_default_size (GTK_WINDOW (assistant),
 	                             600, 500);
-	
+
 	gtk_widget_show_all (GTK_WIDGET (assistant));
-	
+
 	return GTK_WIDGET(assistant);
 }
 
@@ -1143,7 +1192,7 @@ npw_druid_add_default_property (NPWDruid* druid)
 	/* Add default base project directory */
 	value = npw_value_heap_find_value (druid->values, ANJUTA_PROJECT_DIRECTORY_PROPERTY);
 	npw_value_set_value (value, g_get_home_dir (), NPW_VALID_VALUE);
-	
+
 	/* Add user name */
 	value = npw_value_heap_find_value (druid->values, USER_NAME_PROPERTY);
 	s = (gchar *)g_get_real_name();
@@ -1170,7 +1219,7 @@ npw_druid_new (NPWPlugin* plugin, const gchar *directory)
 	{
 		anjuta_util_dialog_error (NULL, _("Could not find autogen version 5; please install the autogen package. You can get it from http://autogen.sourceforge.net."));
 		return NULL;
-	}	
+	}
 
 	druid = g_new0(NPWDruid, 1);
 	druid->plugin = plugin;
@@ -1185,10 +1234,10 @@ npw_druid_new (NPWPlugin* plugin, const gchar *directory)
 	if (npw_druid_create_assistant (druid, directory) == NULL)
 	{
 		npw_druid_free (druid);
-		
+
 		return NULL;
 	}
-	
+
 	npw_druid_add_default_property (druid);
 
 	return druid;
@@ -1197,19 +1246,24 @@ npw_druid_new (NPWPlugin* plugin, const gchar *directory)
 void
 npw_druid_free (NPWDruid* druid)
 {
-	/* NPWPage* page; */
+	NPWPage* page;
 
 	g_return_if_fail (druid != NULL);
 
 	/* Delete page list */
-	druid->next_page = PROPERTY_PAGE;
-	npw_druid_remove_following_page (druid);
+
+	while ((page = (NPWPage *)g_queue_pop_head (druid->page_list)) != NULL)
+	{
+		npw_page_free (page);
+	}
 	g_queue_free (druid->page_list);
 	npw_value_heap_free (druid->values);
 	npw_autogen_free (druid->gen);
 	if (druid->parser != NULL) npw_page_parser_free (druid->parser);
 	npw_header_list_free (druid->header_list);
 	gtk_widget_destroy (GTK_WIDGET (druid->window));
+	g_object_unref (druid->error_page);
+	g_object_unref (druid->progress_page);
 	druid->plugin->druid = NULL;
 	g_free (druid);
 }
