@@ -486,8 +486,7 @@ on_editor_buffer_symbol_update_scan_end (SymbolDBEngine *dbe, gint process_id,
 			/* remove both the items */
 			g_ptr_array_remove_index (sdb_plugin->buffer_update_ids, i);
 			
-			g_ptr_array_remove_index (sdb_plugin->buffer_update_files, 
-													i);
+			g_ptr_array_remove_index (sdb_plugin->buffer_update_files, i);
 
 			/* no need to free the string coz the g_ptr_array is built with
 			 * g_ptr_array_new_with_free_func (g_free)
@@ -1349,10 +1348,10 @@ static gboolean
 do_check_offline_files_changed (SymbolDBPlugin *sdb_plugin)
 {
 	GList * prj_elements_list;
+	GList *node;
 	IAnjutaProjectManager *pm;
 	GHashTable *prj_elements_hash;
 	GPtrArray *to_add_files = NULL;
-	gint i;
 	gint real_added = 0;
 	
 	pm = anjuta_shell_get_interface (ANJUTA_PLUGIN (sdb_plugin)->shell,
@@ -1367,19 +1366,22 @@ do_check_offline_files_changed (SymbolDBPlugin *sdb_plugin)
 	 */
 	prj_elements_hash = g_hash_table_new_full (g_str_hash, g_str_equal,
 	                                           NULL, g_free);
-	
-	for (i = 0; i <  g_list_length (prj_elements_list); i++)
+
+	node = prj_elements_list;
+	while (node != NULL)
 	{	
 		GFile *gfile;
 		gchar *filename;
 
-		gfile = (GFile *)g_list_nth_data (prj_elements_list, i);
-		
+		gfile = node->data;
+	
 		if ((filename = g_file_get_path (gfile)) == NULL || 
 			g_strcmp0 (filename, "") == 0)
 		{
 			if (gfile)
 				g_object_unref (gfile);
+
+			node = g_list_next (node);
 			continue;
 		}
 		
@@ -1388,6 +1390,8 @@ do_check_offline_files_changed (SymbolDBPlugin *sdb_plugin)
 		{
 			if (gfile)
 				g_object_unref (gfile);
+
+			node = g_list_next (node);
 			continue;
 		}
 
@@ -1400,6 +1404,8 @@ do_check_offline_files_changed (SymbolDBPlugin *sdb_plugin)
 			                     filename),
 		                     filename);
 		g_object_unref (gfile);
+
+		node = g_list_next (node);
 	}	
 	
 
@@ -1600,7 +1606,7 @@ on_project_loaded (IAnjutaProjectManager *pm, GError *error,
 
 		DEBUG_PRINT ("Updating project symbols.");
 		/* update any files of the project which isn't up-to-date */
-		if (do_update_project_symbols (sdb_plugin, sdb_plugin->project_root_dir) == FALSE)
+		if (do_update_project_symbols (sdb_plugin, sdb_plugin->project_opened) == FALSE)
 		{
 			DEBUG_PRINT ("no changes. Skipping.");
 		}
